import 'package:code_id_network/code_id_network.dart';
import 'package:dio/io.dart';

import 'package:fpdart/fpdart.dart';

abstract class INetworkService {
  Future<Either<NetworkError, dynamic>> getHttp({
    required String path,
    int retry = 0,
    String? parameter,
    Map<String, dynamic>? queryParameter,
    Map<String, dynamic>? header,
    Function(int, int)? onReceiveProgress,
    RetryEvaluator? evaluator,
  });

  Future<Either<NetworkError, dynamic>> postHttp({
    required String path,
    int retry = 0,
    String? parameter,
    Map<String, dynamic>? queryParameter,
    dynamic content,
    String? contentType,
    Map<String, dynamic>? header,
    Function(int, int)? onSendProgress,
    Function(int, int)? onReceiveProgress,
    RetryEvaluator? evaluator,
  });

  Future<Either<NetworkError, dynamic>> putHttp({
    required String path,
    int retry = 0,
    String? parameter,
    Map<String, dynamic>? queryParameter,
    dynamic content,
    String? contentType,
    Map<String, dynamic>? header,
    Function(int, int)? onSendProgress,
    Function(int, int)? onReceiveProgress,
    RetryEvaluator? evaluator,
  });

  Future<Either<NetworkError, dynamic>> patchHttp({
    required String path,
    int retry = 0,
    String? parameter,
    Map<String, dynamic>? queryParameter,
    dynamic content,
    String? contentType,
    Map<String, dynamic>? header,
    Function(int, int)? onSendProgress,
    Function(int, int)? onReceiveProgress,
    RetryEvaluator? evaluator,
  });

  Future<Either<NetworkError, dynamic>> delete({
    required String deletePath,
    int retry = 0,
    Map<String, dynamic>? queryParameter,
    Map<String, dynamic>? header,
  });

  Future<Either<NetworkError, dynamic>> download({
    required String url,
    required String downloadPath,
    required String fileName,
    Map<String, dynamic>? queryParameter,
    Map<String, dynamic>? header,
    Function(int, int)? onReceiveProgress,
  });

  Future<Either<NetworkError, dynamic>> request({
    required String path,
    int retry = 0,
    String? parameter,
    Map<String, dynamic>? queryParameter,
    dynamic content,
    String? contentType,
    Map<String, dynamic>? header,
    Function(int, int)? onSendProgress,
    Function(int, int)? onReceiveProgress,
    RetryEvaluator? evaluator,
    NetworkOptions? networkOptions,
    Duration? networkReceiveTimeout,
    Duration? networkSendTimeout,
  });

  Future<Either<NetworkError, bool>> checkInternet();

  void addInterceptors(Iterable<Interceptor> interceptors);

  HttpClientAdapter get httpClientAdapter;

  IOHttpClientAdapter get httpDefaultClientAdapter;

  Dio getDio();
}
